% This uses a 2D dataset where we consider 2 categories. We give positive
% item-category affinities for 2 points only, and use a Gaussian affinity
% matrix for the item-item affinities. First we train the LASS model and
% then we apply its out-of-sample mapping to 4 test points, for which we
% only give item-item affinities (no item-category affinities).


% 2-moons dataset
n=2000; t=linspace(0,pi,n)';
X=[cos(t)-1.25,sin(t); cos(t+pi),sin(t+pi)]; X=X+randn(size(X))*0.10;
i1=n; i2=2*n;					% One point from each cluster

figure(1); clf; hold on;
plot(X(:,1), X(:,2),'k.');
plot(X(i1,1),X(i1,2),'r>','MarkerSize',16,'MarkerFaceColor','r');
plot(X(i2,1),X(i2,2),'b>','MarkerSize',16,'MarkerFaceColor','b');
hold off; ax=[-2.65 1.35 -1.5 1.5]; axis(ax); daspect([1 1 1]); box on;


% Set up affinities to train the LASS model
N = size(X,1);
W = gaussaff(X,{'k',5}); L = diag(sparse(sum(W,2)))-W;	% Item-item
K = 2; G = zeros(N,K); G(i1,1) = 1; G(i2,2) = 1;	% Item-category
l = 1;							% Trade-off parameter

% Estimated optimal ADMM penalty parameter. In the 2moons dataset W has 2
% connected components, so the smallest nonzero eigenvalue is evs(1) below.
evs1 = eigs(L,3,'SM'); evs2 = eigs(L,1,'LM'); r = 2*l*sqrt(evs1(1)*evs2);

% Train LASS model
[Z,Y,U,nu] = lass(L,l,G,r,[],[],[],1e4,1e-6);
Z = SimplexProj(Z);					% Ensure feasible point

% Plot assignments
C = Z(:,1)*[1 0 0] + Z(:,2)*[0 0 1];
figure(2); clf; hold on;
for i=1:N plot(X(i,1),X(i,2),'.','Color',C(i,:)); end
plot(X(i1,1),X(i1,2),'k>','MarkerSize',16,'MarkerFaceColor','k');
plot(X(i2,1),X(i2,2),'k>','MarkerSize',16,'MarkerFaceColor','k');
hold off; axis(ax); daspect([1 1 1]); box on;

figure(3); plot(1:N,Z(:,1),'r-',1:N,Z(:,2),'b-');
ylim([-0.05 1.05]); xlabel('n'); ylabel('z_n');
legend('Category 1: z_{n1}','Category 2: z_{n2}');


% Out-of-sample mapping (for simplicity, for training points)
ii = [1 N/4 3*N/4 N];				% Data point indices
w = W(ii,:);					% Item-item affinities
g = zeros(length(ii),K);			% Item-category affinities
z = lassmap(w,l,g,Z)

